require 'spec_helper'

describe Deadlift::Barbell::Base do

  describe 'with params' do
    let(:params) { { weight: '100' } }
    let(:barbell) { SomeBarbell.new(params) }

    specify {
      expect(barbell.approved?).to be_truthy
      expect(barbell.weight).to eq params[:weight]
    }
  end

  describe 'with nested params' do
    let!(:barbell) { DeadliftWithAFriendBarbell.new(params) }

    describe 'valid input' do
      let(:friend_params) { { name: 'Arnold' } }
      let(:params) { { weight: '100', friend: friend_params } }

      specify {
        expect(barbell.approved?).to be_truthy
        expect(barbell.weight).to eq params[:weight]
        expect(barbell.friend.name).to eq friend_params[:name]
      }
    end

    describe 'invalid input' do
      let(:nested_params) { { name: 'Arnold' } }
      let(:params) { { weight: nil, friend: nested_params } }

      specify {
        expect(barbell.approved?).to be_falsey
        expect(barbell.errors[:weight].size).to eq(1)
      }
    end

    describe 'invalid input on nested' do
      let(:nested_params) { { name: nil } }
      let(:params) { { weight: 52, friend: nested_params } }

      specify {
        expect(barbell.approved?).to be_falsey
        expect(barbell.errors[:'friend.name'].size).to eq(1)
      }
    end

    describe 'multiple instances' do
      let(:params) { { weight: 52 } }
      let(:number_two) { DeadliftWithAFriendBarbell.new({}) }

      specify { expect(number_two.weight).to_not eq 52 }
    end
  end
end
