module DeadliftHelpers
  class SuccessCaller

    def initialize(arguments)
      @barbell = OpenStruct.new(arguments)
      @has_performed = false
    end

    def add_trainer(trainer)
      @trainers ||= []
      @trainers << trainer
      self
    end

    def has_performed?
      @has_performed
    end

    def perform
      @has_performed = true
      @trainers.each do |trainer|
        trainer.public_send :success, Deadlifts::Performance.new(@barbell, nil, nil)
      end
    end
  end

  class FailureCaller

    def initialize(arguments)
      @barbell = Deadlifts::DefaultBarbell.new(arguments)
      @barbell.errors.add(:base, :invalid)
      @has_performed = false
    end

    def add_trainer(trainer)
      @trainers ||= []
      @trainers << trainer
      self
    end

    def has_performed?
      @has_performed
    end

    def perform
      @has_performed = true
      @trainers.each do |trainer|
        trainer.public_send :failure, Deadlifts::Performance.new(@barbell, nil, nil, user: SystemUser.new)
      end
    end
  end


  class PerformanceSpy
    attr_reader :performance, :state, :deadlift, :successes, :failures, :progresses

    def initialize
      @successes = 0
      @failures = 0
      @progresses = 0
    end

    def success(performance)
      @deadlift = performance.deadlift
      @performance = performance
      @state = :success
      @successes+=1
    end

    def failure(performance)
      @deadlift = performance.deadlift
      @performance = performance
      @state = :failure
      @failures+=1
    end

    def progress(performance)
      @deadlift = performance.deadlift
      @performance = performance
      @state = :progress
      @progresses+=1
    end

    def result
      @performance.result
    end
  end
end
